# -*- coding: utf-8 -*-
# Copyright 2008-2009, BlueDynamics Alliance, Austria - http://bluedynamics.com
# GNU General Public Licence Version 2 or later

"""Module recurring.

This module provide functions to calculate recurring.

How to get the current day:
    
    from DateTime import DateTime
    from math import floor
    dt = DateTime('2007/03/20 10:00:00')
    # moment results in 13591.0
    moment = floor(float(dt.millis() / float(86400000)))

remarks: millis is always relative to GMT
"""

__author__ = """Pawel Marzec <marzec@cjg.pl>,
                Robert Niederreiter <office@squarewave.at>,
                Jens Klein <jens@bluedynamics.com>"""
__docformat__ = 'plaintext'

import pytz
from datetime import datetime
from datetime import timedelta
from converter import dt2epochday
from calculator import numberOfDaysInMonth
from calculator import numberOfDaysInYear

RO_DAILY = "daily"
RO_MONTHLY = "monthly"
RO_WEEKLY = "weekly"
RO_YEARLY = "yearly"

RM_ONE_TIME = "one time"
RM_UNTIL = "until"
RM_FOREVER = "forever"

# the date in future which we assume as "end of forever"
FROZEN_DATE_IN_FUTURE = datetime(datetime.now().year + 10, 12, 31, 23, 59, 59, 
                                 tzinfo=pytz.timezone('UTC'))
    
# calculators for recue indexes. i.e. for fast search via portal_catalog
def recueDays(start, until, recuemode, offset): 
    """Generates list of recue days.
    
    * start - datetime
    * until - datetime, hour and minute matters!
    * recuemode - from RECUE_MODES
    * offset - current offset
    """
    if recuemode == RM_ONE_TIME:
        days = [dt2epochday(start)]
    elif recuemode == RM_UNTIL:
        days = _generateDays(start, until, offset)
    elif recuemode == RM_FOREVER: 
        days = _generateDays(start, FROZEN_DATE_IN_FUTURE, offset)
    else:
        days = None
    return days

def _recueTimeDeltaDays(dt, offset):
    if offset == RO_DAILY: 
        return timedelta(1)
    if offset == RO_WEEKLY: 
        return timedelta(7)
    if offset == RO_MONTHLY: 
        return timedelta(numberOfDaysInMonth(dt))
    if offset == RO_YEARLY: 
        return timedelta(numberOfDaysInYear(dt))
    raise ValueError, 'Invalid offset type'
    
# private index values generators
def _generateDays(start, end, offset):    
    days = []
    idt = start
    #print "start %s to end %s" % (idt, end)
    while idt <= end:
       days.append(dt2epochday(idt))
       idt = idt + _recueTimeDeltaDays(idt, offset)
    return days  



# -*- coding: utf-8 -*-
# Copyright 2008-2009, BlueDynamics Alliance, Austria - http://bluedynamics.com
# GNU General Public Licence Version 2 or later

from math import floor
import datetime
import pytz
from DateTime import DateTime

ONEDAYINMILLIS = 86400000.0

def dt2DT(dt):
    """Convert Python's datetime to Zope's DateTime.
    
    Acts timezone-aware.
    """
    if isinstance(dt, DateTime):
        return dt
    return DateTime(dt.isoformat())

def DT2dt(DT):
    """Convert Zope's DateTime to Pythons's datetime.
    
     Act timezone-neutral, outcome is on UTC.
    """
    if isinstance(DT, datetime.datetime):
        return DT
    # seconds (parts[6]) is a float, so we map to int
    DTutc = DT.toZone('UTC')
    year, month, day, hour, min, sec = DTutc.parts()[:6]
    sec = int(sec)
    utc = pytz.timezone('UTC')
    dt = datetime.datetime(year, month, day, hour, min, sec, tzinfo=utc)
    return dt

def dt2UTCString(dt):
    """build a string from the timetuple as UTC.
    """
    dt = dt.astimezone(pytz.timezone('UTC'))
    return '-'.join(['%d'%v for v in dt.timetuple()[:6]])

def dtFromUTCString(utcstr):
    """build datetime from timetuple UTC string.
    """
    y, m, d, h, mi, s = utcstr.split('-')
    dt = datetime.datetime(int(y), int(m), int(d), int(h), int(mi), int(s), 
                           tzinfo=pytz.timezone('UTC'))
    return dt

def dt2epochday(dt):
   """Number of days since epoch.
   
   timezone gets a problem here, we need to normalize all to GMT to make it 
   recognize the same day even if it a different timezone:
   i.e. 2008-05-01T00:00:00+02:00 (CEST) 
   """      
   DT = dt2DT(dt) # if possible replace this one and call in next line
   days = DT.earliestTime().millis() / ONEDAYINMILLIS
   idays = floor(days)
   return idays


# -*- coding: utf-8 -*-
# Copyright 2008-2009, BlueDynamics Alliance, Austria - http://bluedynamics.com
# GNU General Public Licence Version 2 or later

from converter import DT2dt
from datetime import datetime
from timezone import timezoneAdjuster

def dtYear(dt, context=None):
    """Return the year of dt.
    """
    if pyDt(dt):
        dt = timezoneAdjuster(context, dt)
        return dt.year
    return dt.year()

def dtMonth(dt, context=None):
    """Return the month of dt.
    """
    if pyDt(dt):
        dt = timezoneAdjuster(context, dt)
        return dt.month
    return dt.month()

def dtDay(dt, context=None):
    """Return the day of dt.
    """
    if pyDt(dt):
        dt = timezoneAdjuster(context, dt)
        return dt.day
    return dt.day()

def dtHour(dt, context=None):
    """Return the hour of dt.
    """
    if pyDt(dt):
        dt = timezoneAdjuster(context, dt)
        return dt.hour
    return dt.hour()

def dtMinute(dt, context=None):
    """Return the minute of dt.
    """
    if pyDt(dt):
        dt = timezoneAdjuster(context, dt)
        return dt.minute
    return dt.minute()

def dtWeekday(dt, context=None):
    """Return the weekday of dt.
    """
    if pyDt(dt, context=None):
        dt = timezoneAdjuster(context, dt)
        return dt.weekday() + 1
    return dt.dow()

def pyDt(dt, context=None):
    """Return wether dt is instance of datetime object.
    """
    return isinstance(dt, datetime)



# -*- coding: utf-8 -*-
# Copyright 2008-2009, BlueDynamics Alliance, Austria - http://bluedynamics.com
# GNU General Public Licence Version 2 or later

import datetime
import calendar

def CWof(date):
    """the calendar week number of a date.
    
    @param date: python datetime
    """
    return date.isocalendar()[1]       

def firstDayOfCW(year, cw, tzinfo=None):
    """first day of a numbered calendar week.
    
    @param year: year where the week is of (int)
    @param cw: calendar week (int).
    """
    # thx to Peter Otten, base found at 
    # http://mail.python.org/pipermail/python-list/2004-May/264156.html
    day = datetime.datetime(year, 2, 1, tzinfo=tzinfo)
    year, weekBase, dayBase = day.isocalendar()
    day += datetime.timedelta(1 - dayBase + (cw - weekBase)*7)
    return day

def daysOfWeek(year, kw, tzinfo=None):
    """yields 7 datetimes of the given calendar week.
    """
    # thx to Peter Otten, base found at 
    # http://mail.python.org/pipermail/python-list/2004-May/264156.html
    day = firstDayOfCW(year, kw, tzinfo) 
    delta = datetime.timedelta(1)
    for i in range(6):
        yield day
        day += delta
    yield day        

def numberOfDaysInYear(dt):
    """calculate number of day of the year of the given datetime"""
    if dt.isLeapYear(): 
        return 366
    return 365

def numberOfDaysInMonth(dt):
    """calculates number of days of the given datetimes month
    """
    firstday, numdays = calendar.monthrange(dt.year, dt.month) 
    return numdays
        
def daysOfMonth(year, month, tzinfo=None):
    """yields all days as datetimes of the given month"""
    day = datetime.datetime(year, month, 1, 0, 0, 0, tzinfo=tzinfo)
    numdays = numberOfDaysInMonth(day)
    delta = datetime.timedelta(1)
    for i in range(numdays-1):
        yield day
        day += delta
    yield day 

def hoursOfDay(year, month, day, tzinfo=None):
    """yields all hours of a day as datetimes.
    """
    hour = datetime.datetime(year, month, day, 0, 0, 0, tzinfo=tzinfo)        
    delta = datetime.timedelta(0, 3600)
    for i in range(23):
        yield hour
        hour += delta
    yield hour 

def isSameDay(first, second):
    """detects if two dates are the same day. bool."""
    return first.day == second.day and \
           first.month == second.month and \
           first.year == second.year



from setuptools import setup, find_packages
import sys, os

version = '1.2.2'

shortdesc = "Base common calendaring features: Convinience or not coverd yet."
longdesc = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

setup(name='bda.calendar.base',
      version=version,
      description=shortdesc,
      long_description=longdesc,
      classifiers=[
            'Development Status :: 4 - Beta',
            'Environment :: Web Environment',
            'Framework :: Zope2',
            'License :: OSI Approved :: GNU General Public License (GPL)',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            'Topic :: Internet :: WWW/HTTP :: Dynamic Content',            
      ], # http://www.python.org/pypi?%3Aaction=list_classifiers
      keywords='Calendaring',
      author='Jens Klein, Robert Niedereiter',
      author_email='dev@bluedynamics.com',
      url=u'http://svn.plone.org/svn/collective/bda.calendar.base',
      license='General Public Licence',
      packages=find_packages('src'),
      package_dir = {'': 'src'},
      namespace_packages=['bda', 'bda.calendar'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'pytz',
          'zope.interface',              
          # -*- Extra requirements: -*
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )


# -*- coding: utf-8 -*-
# Copyright 2008-2009, BlueDynamics Alliance, Austria - http://bluedynamics.com
# GNU General Public Licence Version 2 or later

from zope.interface import Interface

class ITimezoneFactory(Interface):
    """Find the current timezone as pytz timezone."""

    def __call__():
        """perform the detection."""
        



# -*- coding: utf-8 -*-
# Copyright 2008-2009, BlueDynamics Alliance, Austria - http://bluedynamics.com
# GNU General Public Licence Version 2 or later

import time
import pytz
from datetime import datetime
from zope.interface import implementer
from interfaces import ITimezoneFactory

@implementer(ITimezoneFactory)
def ServerTimezoneFactory(context):
    """The timezone of the server."""
    zones =  time.tzname
    if zones and len(zones) > 0:
        zone = zones[0]
    else:
        zone = 'UTC'
    return pytz.timezone(zone)
    
def timezoneAdjuster(context, dt):
    """convinience: new datetime with given timezone."""
    newtz = ITimezoneFactory(context)
    return dt.astimezone(newtz)

def tzawarenow():
    now = datetime.utcnow()
    return datetime(now.year, now.month, now.day, now.hour, now.minute,
                    now.second, tzinfo=pytz.timezone('UTC'))